%% Load input images, mask, and groundtruth lighting S
load('korean_doll.mat','ims','mask','S');
%load('redfish.mat','ims','mask','S');
%load('octopus.mat','ims','mask','S');

% Convert ims to grayscale
ims_gray = cellfun(@(x)(mean(x,3)), ims, 'UniformOutput',false);

%% Perform calibrated photometric stereo on the images to compute the normal/albedo maps (for ground truth)
B = calibrated_photometric_stereo(ims_gray,S,mask(:));

%% Perform uncalibrated photometric stereo
gbrinit = [1,1,1];
[uB,uS] = uncalibrated_photometric_stereo(ims_gray,mask,gbrinit);

% Swap the sign of the z-compenent if it is negative on average (normals should
%  point toward the camera...
if sum(uB(:,3)) < 0
	uB = uB * [1,0,0;0,1,0;0,0,-1];
	uS = [1,0,0;0,1,0;0,0,-1] * uS;
end

%% Compute the ground-truth GBR transform such that B = uB*G and G*S = uS
G_gt = uS / S;
%  Force G to nearest valid GBR using procrustes method
%   G = [a,0,0; 0,a,0; a*mu,a*nu, a*lambda]
a = (G_gt(1,1)+G_gt(2,2))/2;
G_gt = [a,0,0; 0,a,0; G_gt(3,:)];

%% Solve for GBR parameters by minimizing the albedo entropy
[mu,nu,lambda,e] = solve_gbr(uB(mask(:),:),uS);

G_me = [ ...
	1, 0, 0; ...
	0, 1, 0;
	mu, nu, lambda];

%% Fix global scale factor to be the same as groundtruth (for visual comparison)
G_me = G_me * G_gt(1,1);

%% Plot surfaces resulting from calibrated PS and entropy minimization
B_gt = uB*G_gt;
B_me = uB*G_me;
figure; [hmap_ps] = plot_Bsurf(B,size(ims_gray{1}),mask); title('Calibrated PS');
figure; [hmap_uc] = plot_Bsurf(uB,size(ims_gray{1}),mask); title('Uncalibrated PS');
figure; [hmap_gt] = plot_Bsurf(uB*G_gt,size(ims_gray{1}),mask); title('Ground Truth');
figure; [hmap_me] = plot_Bsurf(uB*G_me,size(ims_gray{1}),mask); title('Min. Entropy');

%% Plot albedo distributions
[nx ny nz a] = B2normals(B,size(ims_gray{1}));
[nx_uc ny_uc nz_uc a_uc] = B2normals(uB,size(ims_gray{1}));
[nx_gt ny_gt nz_gt a_gt] = B2normals(B_gt,size(ims_gray{1}));
[nx_me ny_me nz_me a_me] = B2normals(B_me,size(ims_gray{1}));

figure; hist(a(mask(:)),400); title('Calibrated PS');
figure; hist(a_uc(mask(:)),400); title('Uncalibrated PS');
figure; hist(a_gt(mask(:)),400); title('Ground Truth');
figure; hist(a_me(mask(:)),400); title('Min. Entropy');

%% Plot normal/albedo maps
%  To get a colored albedo map, project the albedo intensity onto the average color of the input images
img_chroma = sum(cat(4,ims{:}),4);
img_chroma = img_chroma ./ repmat(max(eps,mean(img_chroma,3)),[1,1,size(img_chroma,3)]);
ac = repmat(a,[1,1,size(img_chroma,3)]).*img_chroma / max(a(:));
ac_uc = repmat(a_uc,[1,1,size(img_chroma,3)]).*img_chroma;
ac_gt = repmat(a_gt,[1,1,size(img_chroma,3)]).*img_chroma;
ac_me = repmat(a_me,[1,1,size(img_chroma,3)]).*img_chroma;

ac = imadjust(ac,stretchlim(a,[0,.98]));
ac_uc = imadjust(ac_uc,stretchlim(a_uc,[0,.98]));
ac_gt = imadjust(ac_gt,stretchlim(a_gt,[0,.98]));
ac_me = imadjust(ac_me,stretchlim(a_me,[0,.98]));

figure; imshow([colorize_data([nx.*mask ny.*mask nz.*mask]) ac.*repmat(mask,[1,1,3])]); title('Calibrated PS');
figure; imshow([colorize_data([nx_uc.*mask ny_uc.*mask nz_uc.*mask]) ac_uc.*repmat(mask,[1,1,3])]); title('Uncalibrated PS');
figure; imshow([colorize_data([nx_gt.*mask ny_gt.*mask nz_gt.*mask]) ac_gt.*repmat(mask,[1,1,3])]); title('Ground Truth');
figure; imshow([colorize_data([nx_me.*mask ny_me.*mask nz_me.*mask]) ac_me.*repmat(mask,[1,1,3])]); title('Min. Entropy');
